
# q-expansion of newform 1160.2.a.b, downloaded from the LMFDB on 15 January 2025.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1160
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [0], [1], [0], [0], [-2], [-6], [-8], [0], [1], [4], [-2], [-6], [8], [4], [-10], [4], [-2], [-12], [-8], [2], [4], [12], [-6], [10], [6], [0], [4], [-18], [-6], [-4], [0], [2], [-12], [-10], [-8], [-10], [24], [8], [-2], [20], [22], [4], [-6], [22], [-16], [0], [24], [12], [22], [10], [-24], [-14], [0], [2], [-12], [-18], [-20], [-10], [26], [28], [30], [-16], [12], [-6], [6], [-8], [-22], [12], [-2], [-14], [-20], [-4], [-26], [-32], [-16], [22], [-2], [18], [-6], [20], [22], [-24], [-6], [-40], [32], [-14], [26], [-18], [-32], [16], [-4], [-32], [-8], [4], [28], [-34], [-6], [-28], [-18], [-4], [-18], [24], [-6], [-28], [42], [-12], [-30], [12], [-6], [-28], [6], [18], [24], [-24], [-30], [-44], [24], [6], [40], [22], [-46], [-18], [-28], [-20], [-2], [38], [-24], [-12], [6], [-8], [44], [-4], [-34], [10], [2], [-50], [-28], [54], [-6], [28], [-42], [-44], [0], [46], [-28], [14], [-6], [24], [16], [-18], [2], [20], [20], [48], [44], [-16], [50], [-38], [14], [-48], [-6], [44], [48], [18], [-36], [-16], [-2], [34], [-18], [12], [-34], [-48], [-14], [24], [10], [-12], [-42], [4], [46], [-4], [16], [-42], [10], [56], [-10], [-42], [0], [10], [-56], [-14], [8], [8], [-2], [8], [-38], [2], [14], [-14], [-24], [-34], [-16], [14], [-14], [56], [14], [20], [-36], [58], [40], [-38], [-42], [28], [-24], [60], [-38], [48], [2], [-24], [-42], [54], [48], [-30], [-36], [-12], [22], [-30], [12], [28], [-44], [-10], [-28], [68], [-54], [-20], [4], [-62], [-66], [68], [12], [-8], [-12], [-24], [-18], [-14], [40], [56], [52], [-20], [12], [-74], [-46], [60], [58], [-42], [-36], [22], [-24], [38], [42], [-12], [-48], [38], [-10], [42], [72], [-18], [-54], [-16], [-26], [-50], [-52], [74], [-76], [-6], [-12], [-48], [30], [74], [-28], [-56], [-24], [48], [-26], [-40], [-20], [-70], [46], [0], [34], [-50], [84], [42], [32], [6], [30], [-52], [86], [-44], [8], [34], [-18], [-36], [48], [8], [-62], [-48], [14], [24], [22], [0], [-26], [-62], [44], [-40], [42], [8], [-56], [-30], [-78], [8], [-38], [-2], [20], [10], [50], [84], [28], [60], [6], [14], [-82], [32], [8], [88], [-52], [-34], [42], [10], [56], [46], [42], [-10], [80], [78], [-24], [22], [32], [-28], [62], [-76], [58], [-18], [16], [-58], [18], [-92], [-20], [74], [16], [86], [42], [28], [-12], [-64], [-46], [-74], [-56], [-86], [-48], [-76], [-24], [22], [-8], [62], [16], [60], [42], [-62], [26], [-50], [74], [92], [-22], [-20], [-64], [100], [-58], [32], [92], [66], [38], [48], [56], [12], [26], [80], [-70], [12], [6], [-34], [42], [84], [-54], [-90], [16], [-18], [98], [16], [-60], [18], [22], [4], [52], [-62], [-34], [-36], [64], [58], [-60], [-66], [-34], [-12], [72], [106], [14], [-12], [74], [-4], [-12]]
